"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const dotenv_1 = __importDefault(require("dotenv"));
const web3_1 = __importDefault(require("web3"));
require("mocha");
const moment_1 = __importDefault(require("moment"));
const origin_backend_client_mocks_1 = require("@energyweb/origin-backend-client-mocks");
const migrate_1 = require("../migrate");
const __1 = require("..");
const Logger_1 = require("../Logger");
describe('Cerificate tests', () => {
    let conf;
    let issuer;
    dotenv_1.default.config({
        path: '.env.test'
    });
    const web3 = new web3_1.default(process.env.WEB3);
    const deviceOwnerPK = '0x622d56ab7f0e75ac133722cc065260a2792bf30ea3265415fe04f3a2dba7e1ac';
    const accountDeviceOwner = web3.eth.accounts.privateKeyToAccount(deviceOwnerPK).address;
    const issuerPK = '0x50397ee7580b44c966c3975f561efb7b58a54febedaa68a5dc482e52fb696ae7';
    const issuerAccount = web3.eth.accounts.privateKeyToAccount(issuerPK).address;
    const traderPK = '0xca77c9b06fde68bcbcc09f603c958620613f4be79f3abb4b2032131d0229462e';
    const accountTrader = web3.eth.accounts.privateKeyToAccount(traderPK).address;
    let timestamp = moment_1.default()
        .subtract(10, 'year')
        .unix();
    const setActiveUser = (privateKey) => {
        conf.blockchainProperties.activeUser = {
            address: web3.eth.accounts.privateKeyToAccount(privateKey).address,
            privateKey
        };
    };
    const issueCertificate = async (volume, isPrivate = false) => {
        setActiveUser(issuerPK);
        const generationStartTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const generationEndTime = timestamp;
        const deviceId = '1';
        return __1.Certificate.createCertificate(accountDeviceOwner, volume, generationStartTime, generationEndTime, deviceId, conf, isPrivate);
    };
    it('migrates Registry', async () => {
        const registry = await migrate_1.migrateRegistry(web3, issuerPK);
        issuer = await migrate_1.migrateIssuer(web3, issuerPK, registry.web3Contract.options.address);
        conf = {
            blockchainProperties: {
                activeUser: {
                    address: issuerAccount,
                    privateKey: issuerPK
                },
                registry,
                issuer,
                web3
            },
            offChainDataSource: new origin_backend_client_mocks_1.OffChainDataSourceMock(),
            logger: Logger_1.logger
        };
    });
    it('gets all certificates', async () => {
        const totalVolume = 1e9;
        await issueCertificate(totalVolume);
        await issueCertificate(totalVolume);
        const allCertificates = await __1.Certificate.getAllCertificates(conf);
        chai_1.assert.equal(allCertificates.length, 2);
    });
    it('issuer issues a certificate', async () => {
        const volume = 1e9;
        const certificate = await issueCertificate(volume);
        chai_1.assert.isNotNull(certificate.id);
        setActiveUser(issuerPK);
        chai_1.assert.isFalse(certificate.isOwned());
        chai_1.assert.equal(certificate.energy, volume);
        setActiveUser(deviceOwnerPK);
        chai_1.assert.isTrue(certificate.isOwned());
        chai_1.assert.equal(certificate.ownedVolume().publicVolume, volume);
        chai_1.assert.deepOwnInclude(certificate, {
            initialized: true,
            issuer: issuer.web3Contract.options.address
        });
    });
    it('transfers a certificate', async () => {
        const totalVolume = 1e9;
        let certificate = await issueCertificate(totalVolume);
        setActiveUser(deviceOwnerPK);
        await certificate.transfer(accountTrader, totalVolume / 4);
        certificate = await certificate.sync();
        chai_1.assert.isTrue(certificate.isOwned(accountDeviceOwner));
        chai_1.assert.equal(certificate.ownedVolume(accountDeviceOwner).publicVolume, (totalVolume / 4) * 3);
        chai_1.assert.isTrue(certificate.isOwned(accountTrader));
        chai_1.assert.equal(certificate.ownedVolume(accountTrader).publicVolume, totalVolume / 4);
    });
    it('transfers a private certificate', async () => {
        const totalVolume = 1e9;
        let certificate = await issueCertificate(totalVolume, true);
        setActiveUser(deviceOwnerPK);
        await certificate.transfer(accountTrader, totalVolume, true);
        certificate = await certificate.sync();
        setActiveUser(issuerPK);
        await certificate.approvePrivateTransfer();
        certificate = await certificate.sync();
        chai_1.assert.isFalse(certificate.isOwned(accountDeviceOwner));
        chai_1.assert.equal(certificate.ownedVolume(accountDeviceOwner).privateVolume, 0);
        chai_1.assert.isTrue(certificate.isOwned(accountTrader));
        chai_1.assert.equal(certificate.ownedVolume(accountTrader).privateVolume, totalVolume);
    });
    it('partially transfers a private certificate', async () => {
        const totalVolume = 1e9;
        const partialVolumeToSend = totalVolume / 4;
        let certificate = await issueCertificate(totalVolume, true);
        setActiveUser(deviceOwnerPK);
        await certificate.transfer(accountTrader, partialVolumeToSend, true);
        certificate = await certificate.sync();
        setActiveUser(issuerPK);
        await certificate.approvePrivateTransfer();
        certificate = await certificate.sync();
        chai_1.assert.isTrue(certificate.isOwned(accountDeviceOwner));
        chai_1.assert.equal(certificate.ownedVolume(accountDeviceOwner).privateVolume, totalVolume - partialVolumeToSend);
        chai_1.assert.isTrue(certificate.isOwned(accountTrader));
        chai_1.assert.equal(certificate.ownedVolume(accountTrader).privateVolume, partialVolumeToSend);
    });
    it('fails claiming a revoked certificate', async () => {
        const totalVolume = 1e9;
        const certificate = await issueCertificate(totalVolume);
        setActiveUser(issuerPK);
        await certificate.revoke();
        setActiveUser(deviceOwnerPK);
        let failed = false;
        try {
            await certificate.claim(totalVolume);
        }
        catch (e) {
            failed = true;
        }
        chai_1.assert.isTrue(failed);
    });
    it('claims a certificate', async () => {
        const totalVolume = 1e9;
        let certificate = await issueCertificate(totalVolume);
        setActiveUser(deviceOwnerPK);
        const amountToSendToTrader = totalVolume / 4;
        await certificate.transfer(accountTrader, amountToSendToTrader);
        certificate = await certificate.sync();
        setActiveUser(traderPK);
        await certificate.claim(amountToSendToTrader);
        certificate = await certificate.sync();
        chai_1.assert.isFalse(certificate.isOwned());
        chai_1.assert.equal(certificate.ownedVolume().publicVolume, 0);
        chai_1.assert.isTrue(certificate.isClaimed());
        chai_1.assert.equal(certificate.claimedVolume(), amountToSendToTrader);
    });
    it('claims a private certificate', async () => {
        const totalVolume = 1e9;
        let certificate = await issueCertificate(totalVolume, true);
        setActiveUser(deviceOwnerPK);
        await certificate.requestMigrateToPublic();
        certificate = await certificate.sync();
        setActiveUser(issuerPK);
        await certificate.migrateToPublic();
        certificate = await certificate.sync();
        setActiveUser(deviceOwnerPK);
        await certificate.claim();
        certificate = await certificate.sync();
        chai_1.assert.isTrue(await certificate.isClaimed());
        chai_1.assert.equal(await certificate.claimedVolume(), totalVolume);
    });
    it('claims a partial private certificate', async () => {
        const totalVolume = 1e9;
        const partialVolumeToClaim = totalVolume / 4;
        let certificate = await issueCertificate(totalVolume, true);
        setActiveUser(deviceOwnerPK);
        await certificate.transfer(accountTrader, partialVolumeToClaim, true);
        certificate = await certificate.sync();
        setActiveUser(issuerPK);
        await certificate.approvePrivateTransfer();
        certificate = await certificate.sync();
        setActiveUser(traderPK);
        await certificate.requestMigrateToPublic();
        certificate = await certificate.sync();
        setActiveUser(issuerPK);
        await certificate.migrateToPublic();
        certificate = await certificate.sync();
        setActiveUser(traderPK);
        await certificate.claim();
        certificate = await certificate.sync();
        chai_1.assert.isTrue(await certificate.isClaimed());
        chai_1.assert.equal(await certificate.claimedVolume(), partialVolumeToClaim);
    });
    it('batch transfers certificates', async () => {
        const totalVolume = 1e9;
        let certificate = await issueCertificate(totalVolume);
        let certificate2 = await issueCertificate(totalVolume);
        setActiveUser(deviceOwnerPK);
        chai_1.assert.isTrue(certificate.isOwned());
        chai_1.assert.isTrue(certificate2.isOwned());
        chai_1.assert.equal(certificate.ownedVolume().publicVolume, totalVolume);
        chai_1.assert.equal(certificate2.ownedVolume().publicVolume, totalVolume);
        await __1.Certificate.transferCertificates([certificate.id, certificate2.id], accountTrader, conf);
        certificate = await certificate.sync();
        certificate2 = await certificate2.sync();
        chai_1.assert.isFalse(certificate.isOwned());
        chai_1.assert.isFalse(certificate2.isOwned());
        chai_1.assert.equal(certificate.ownedVolume().publicVolume, 0);
        chai_1.assert.equal(certificate2.ownedVolume().publicVolume, 0);
        chai_1.assert.isTrue(certificate.isOwned(accountTrader));
        chai_1.assert.isTrue(certificate2.isOwned(accountTrader));
        chai_1.assert.equal(certificate.ownedVolume(accountTrader).publicVolume, totalVolume);
        chai_1.assert.equal(certificate2.ownedVolume(accountTrader).publicVolume, totalVolume);
    });
    xit('batch claims certificates', async () => {
        const totalVolume = 1e9;
        let certificate = await issueCertificate(totalVolume);
        let certificate2 = await issueCertificate(totalVolume);
        setActiveUser(deviceOwnerPK);
        chai_1.assert.isFalse(certificate.isClaimed());
        chai_1.assert.isFalse(certificate2.isClaimed());
        chai_1.assert.equal(certificate.claimedVolume(), 0);
        chai_1.assert.equal(certificate2.claimedVolume(), 0);
        await __1.Certificate.claimCertificates([certificate.id, certificate2.id], conf);
        certificate = await certificate.sync();
        certificate2 = await certificate2.sync();
        chai_1.assert.isTrue(certificate.isClaimed());
        chai_1.assert.isTrue(certificate2.isClaimed());
        chai_1.assert.equal(certificate.claimedVolume(), totalVolume);
        chai_1.assert.equal(certificate2.claimedVolume(), totalVolume);
    });
});
//# sourceMappingURL=Certificate.test.js.map