"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const dotenv_1 = __importDefault(require("dotenv"));
const web3_1 = __importDefault(require("web3"));
require("mocha");
const moment_1 = __importDefault(require("moment"));
const origin_backend_client_mocks_1 = require("@energyweb/origin-backend-client-mocks");
const migrate_1 = require("../migrate");
const __1 = require("..");
const Logger_1 = require("../Logger");
describe('Issuer', () => {
    let issuer;
    let registry;
    let conf;
    dotenv_1.default.config({
        path: '.env.test'
    });
    const web3 = new web3_1.default(process.env.WEB3);
    const deviceOwnerPK = '0xd9bc30dc17023fbb68fe3002e0ff9107b241544fd6d60863081c55e383f1b5a3';
    const accountDeviceOwner = web3.eth.accounts.privateKeyToAccount(deviceOwnerPK).address;
    const issuerPK = '0x50397ee7580b44c966c3975f561efb7b58a54febedaa68a5dc482e52fb696ae7';
    const issuerAccount = web3.eth.accounts.privateKeyToAccount(issuerPK).address;
    let timestamp = moment_1.default()
        .subtract(10, 'year')
        .unix();
    const setActiveUser = (privateKey) => {
        conf.blockchainProperties.activeUser = {
            address: web3.eth.accounts.privateKeyToAccount(privateKey).address,
            privateKey
        };
    };
    const createCertificationRequest = async (conf, energy, isPrivate = false) => {
        setActiveUser(deviceOwnerPK);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        return __1.CertificationRequest.createCertificationRequest(fromTime, toTime, energy, device, conf, [], isPrivate);
    };
    it('migrates Issuer and Registry', async () => {
        registry = await migrate_1.migrateRegistry(web3, issuerPK);
        issuer = await migrate_1.migrateIssuer(web3, issuerPK, registry.web3Contract.options.address);
        const version = await issuer.version();
        chai_1.assert.equal(version, 'v0.1');
        const registryAddress = await issuer.getRegistryAddress();
        chai_1.assert.equal(registryAddress, registry.web3Contract.options.address);
        conf = {
            blockchainProperties: {
                activeUser: {
                    address: issuerAccount,
                    privateKey: issuerPK
                },
                registry,
                issuer,
                web3
            },
            offChainDataSource: new origin_backend_client_mocks_1.OffChainDataSourceMock(),
            logger: Logger_1.logger
        };
    });
    it('user correctly requests issuance', async () => {
        setActiveUser(deviceOwnerPK);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        const certificationRequest = await __1.CertificationRequest.createCertificationRequest(fromTime, toTime, 1e9, device, conf, []);
        chai_1.assert.isAbove(certificationRequest.id, -1);
        chai_1.assert.deepOwnInclude(certificationRequest, {
            initialized: true,
            deviceId: device,
            owner: accountDeviceOwner,
            fromTime,
            toTime,
            approved: false,
            isPrivate: false
        });
    });
    it('issuer correctly approves issuance', async () => {
        const volume = 1e9;
        let certificationRequest = await createCertificationRequest(conf, volume);
        setActiveUser(issuerPK);
        const certificateId = await certificationRequest.approve();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.approved);
        const deviceOwnerBalance = await registry.balanceOf(accountDeviceOwner, certificateId);
        chai_1.assert.equal(deviceOwnerBalance, volume);
    });
    it('issuer revokes a certificate', async () => {
        const volume = 1e9;
        let certificationRequest = await createCertificationRequest(conf, volume);
        setActiveUser(issuerPK);
        const certificateId = await certificationRequest.approve();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.approved);
        chai_1.assert.isFalse(certificationRequest.revoked);
        await certificationRequest.revoke();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.revoked);
        const deviceOwnerBalance = await registry.balanceOf(accountDeviceOwner, Number(certificateId));
        chai_1.assert.equal(deviceOwnerBalance, volume);
    });
    it('should fail to request 2 certificates with the same generation period', async () => {
        setActiveUser(deviceOwnerPK);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        await __1.CertificationRequest.createCertificationRequest(fromTime, toTime, 1e9, device, conf, []);
        let failed = false;
        try {
            await __1.CertificationRequest.createCertificationRequest(fromTime, toTime, 1e9, device, conf, []);
        }
        catch (e) {
            failed = true;
        }
        chai_1.assert.isTrue(failed);
    });
    it('should request the same certificate after revoking one', async () => {
        setActiveUser(deviceOwnerPK);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        const volume = 1e9;
        let certificationRequest = await __1.CertificationRequest.createCertificationRequest(fromTime, toTime, volume, device, conf, []);
        setActiveUser(issuerPK);
        await certificationRequest.approve();
        certificationRequest = await certificationRequest.sync();
        await certificationRequest.revoke();
        certificationRequest = await certificationRequest.sync();
        const newCertificationRequest = await __1.CertificationRequest.createCertificationRequest(fromTime, toTime, volume, device, conf, []);
        chai_1.assert.exists(newCertificationRequest);
    });
    it('user correctly requests private issuance', async () => {
        const volume = 1e9;
        const certificationRequest = await createCertificationRequest(conf, volume, true);
        chai_1.assert.isAbove(Number(certificationRequest.id), -1);
        chai_1.assert.deepOwnInclude(certificationRequest, {
            initialized: true,
            deviceId: '1',
            owner: accountDeviceOwner,
            approved: false,
            isPrivate: true,
            energy: volume
        });
    });
    it('issuer correctly approves private issuance', async () => {
        const volume = 1e9;
        let certificationRequest = await createCertificationRequest(conf, volume, true);
        setActiveUser(issuerPK);
        const certificateId = await certificationRequest.approve();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.approved);
        const deviceOwnerBalance = await registry.balanceOf(accountDeviceOwner, Number(certificateId));
        chai_1.assert.equal(deviceOwnerBalance, 0);
    });
});
//# sourceMappingURL=Issuer.test.js.map